import 'dart:typed_data';
import 'package:pedantic/pedantic.dart';
import 'package:test/test.dart';
import 'package:dart_nats_client/dart_nats_client.dart';

void main() {
  group('all', () {
    test('simple', () async {
      var client = Client();
      await client.connect('localhost', retryInterval: 1);
      var sub = client.sub('subject1');
      client.pub('subject1', Uint8List.fromList('message1'.codeUnits));
      var msg = await sub.poll();

      // Terminate
      client.close();
      expect(String.fromCharCodes(msg.data), equals('message1'));
    });
    test('respond', () async {
      var server = Client();
      await server.connect('localhost');
      var service = server.sub('service');
      service.getStream().listen((m) {
        m.respondString('respond');
      });

      var requester = Client();
      await requester.connect('localhost');
      var inbox = newInbox();
      var inboxSub = requester.sub(inbox);

      requester.pubString('service', 'request', replyTo: inbox);

      var receive = await inboxSub.poll();

      // Terminate
      requester.close();
      server.close();
      expect(receive.string, equals('respond'));
    });
    test('resquest', () async {
      var server = Client();
      await server.connect('localhost');
      var service = server.sub('service');
      unawaited(service.poll().then((m) {
        m.respond(Uint8List.fromList('respond'.codeUnits));
      }));

      var client = Client();
      await client.connect('localhost');
      var receive = await client.request(
          'service', Uint8List.fromList('request'.codeUnits));

      // Terminate
      server.close();
      client.close();
      expect(receive.string, equals('respond'));
    });
    test('repeat resquest', () async {
      var server = Client();
      await server.connect('localhost');
      var service = server.sub('service');
      service.getStream().listen((m) {
        m.respond(Uint8List.fromList('respond'.codeUnits));
      });

      var client = Client();
      await client.connect('localhost');
      var receive = await client.request(
          'service', Uint8List.fromList('request'.codeUnits));
      receive = await client.request(
          'service', Uint8List.fromList('request'.codeUnits));
      receive = await client.request(
          'service', Uint8List.fromList('request'.codeUnits));
      receive = await client.request(
          'service', Uint8List.fromList('request'.codeUnits));

      // Terminate
      server.close();
      client.close();
      expect(receive.string, equals('respond'));
    });
  });
}
